/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.attributes.AttributeGroupStyleParser");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

// Here's an example of a document that this class is responsible for parsing:
// <style:attributeGroupStyle
//   xmlns:style="http://schema.ibm.com/rdm/2008/attribute/style"
//   description="Features that enable capabilities for the system."
//   displayName="Feature"
//   id="FEAT"
//   namespace="http://com.ibm.rdm.attribute.group.FEAT">
//   <style:validContentType name="application/x-com.ibm.rdm.requirement+xml" default="true"/>
//   <style:enumerationAttributeStyle displayName="Type" id="Type_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="Functional" id="Functional_ID"/>
//     <style:enumerationLiteralStyle displayName="Non-functional" id="Non-functional_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:enumerationAttributeStyle displayName="Priority" id="Priority_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="Must" id="Must_ID"/>
//     <style:enumerationLiteralStyle displayName="Should" id="Should_ID"/>
//     <style:enumerationLiteralStyle displayName="Could" id="Could_ID"/>
//     <style:enumerationLiteralStyle displayName="Won't" id="Won't_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:enumerationAttributeStyle displayName="Status" id="Status_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="Proposed" id="Proposed_ID"/>
//     <style:enumerationLiteralStyle displayName="Approved" id="Approved_ID"/>
//     <style:enumerationLiteralStyle displayName="Incorporated" id="Incorporated_ID"/>
//     <style:enumerationLiteralStyle displayName="Validated" id="Validated_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:enumerationAttributeStyle displayName="Difficulty" id="Difficulty_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="High" id="High_ID"/>
//     <style:enumerationLiteralStyle displayName="Medium" id="Medium_ID"/>
//     <style:enumerationLiteralStyle displayName="Low" id="Low_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:enumerationAttributeStyle displayName="Stability" id="Stability_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="High" id="High_ID"/>
//     <style:enumerationLiteralStyle displayName="Medium" id="Medium_ID"/>
//     <style:enumerationLiteralStyle displayName="Low" id="Low_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:enumerationAttributeStyle displayName="Origin" id="Origin_ID" type="Enum">
//     <style:enumerationLiteralStyle displayName="Help Desk" id="Help Desk_ID"/>
//     <style:enumerationLiteralStyle displayName="Partners" id="Partners_ID"/>
//     <style:enumerationLiteralStyle displayName="Competitors" id="Competitors_ID"/>
//     <style:enumerationLiteralStyle displayName="Customer Visits" id="Customer Visits_ID"/>
//   </style:enumerationAttributeStyle>
//   <style:attributeStyle displayName="Contact" id="Contact_ID" type="String"/>
// </style:attributeGroupStyle>


dojo.declare(
    "com.ibm.rdm.web.attributes.AttributeGroupStyleParser",
    null,
{
    // summary:
    //    This class is responsible for parsing the information found in an
    //    "attribute group style" XSD document.

    STYLE_NAMESPACE: "http://schema.ibm.com/rdm/2008/attribute/style",
    STYLE_QUALIFIER: "style",

    ///////////////////////////////////////////////////////////////////////////
    // Private fields

    _attributeGroupStyleId: null,          // E.g. "FEAT"
    _attributeGroupStyleDisplayName: null, // E.g. "Feature"
    _attributeGroupStyleNamespace: null,   // E.g. "http://com.ibm.rdm.attribute.group.FEAT"

    // _fieldProperties: an array (which also serves as a hashmap)
    //    This array holds an ordered list of the "fields" that are to be
    //    displayed for the attribute group style document this parser is
    //    associated with.
    //
    //    In addition to being an array, this object also serves has a hashmap
    //    in which the key is the field id and the value is the "field".
    //    Similarly, for fields that represent enums, the field's enumValues
    //    array also serves as a hashmap in which the key is the field value id
    //    and the value is an enum value properties list (containing the enum
    //    value's id and displayName).
    //
    //    Here are some examples of the kinds of "fields" that are stored
    //    by this object:
    //
    //    _fieldProperties["Priority_ID"] returns:
    //        {
    //            id: "Priority_ID",
    //            type: "Enum",
    //            displayName: "Priority",
    //            enumValues: [
    //                { id: "Must_ID",   displayName: "Must" },
    //                { id: "Should_ID", displayName: "Should" },
    //                { id: "Could_ID",  displayName: "Could" },
    //                { id: "Won't_ID",  displayName: "Won't" }
    //            ]
    //        }
    //
    //    _fieldProperties["Contact_ID"] returns:
    //        {
    //            id: "Contact_ID",
    //            type: "String",
    //            displayName: "Contact"
    //        }
    //
    //    In this last example, "Contact", is of type "String" but field
    //    types that share this same format include "Boolean", "Date",
	//    "Decimal", "Float", "Integer" and "URI" (i.e. all of them would
	//    have properties only for "id", "type" and "displayName"
	//    [not "enumValues"]).
	//
	//    Field properties that are of type "Enum" represent the enumValues
	//    property via an array object.  However, this array also serves as
	//    a hashmap to enable you to do something like:
	//        _fieldProperties["Priority_ID"].enumValues["Must_ID"]
	//    to obtain the enumValue pertaining to "Must":
	//        { id: "Must_ID",   displayName: "Must" }
	//
	//    The "type" property value will be set to one of the following strings:
	//        Boolean, Date, Decimal, Enum, Float, Integer, String, URI
    _fieldProperties: null,

    ///////////////////////////////////////////////////////////////////////////
    // API

    constructor: function(keywordArgs) {
        // keywordArgs.doc: an AttributeGroupStyle XSD document
        this._attributeGroupStyleId = "";
        this._attributeGroupStyleDisplayName = "";
        this._attributeGroupStyleNamespace = "";
        this._fieldProperties = [];
        this._parseXSD(keywordArgs.doc);
    },

    getId: function() {
        return this._attributeGroupStyleId;
    },

    getDisplayName: function() {
        return this._attributeGroupStyleDisplayName;
    },

    getNamespace: function() {
        return this._attributeGroupStyleNamespace;
    },

    getFieldProperties: function() {
        // summary:
        //    Get the field properties found in the XSD associated with this
		//    class.
        // returns:
        //    An array (which is also a hashmap, see above _fieldProperties
        //    comment for more information).
        //    The elements in this array are ordered in the way that they
        //    should be displayed.
        return this._fieldProperties;
    },

    ///////////////////////////////////////////////////////////////////////////
    // Helpers

    _parseXSD: function(doc) {
        var attrGroupStyleNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
            doc, "attributeGroupStyle",
            this.STYLE_NAMESPACE, this.STYLE_QUALIFIER)[0];
        this._attributeGroupStyleId = attrGroupStyleNode.getAttribute("id");
        this._attributeGroupStyleDisplayName =
			attrGroupStyleNode.getAttribute("displayName");
        this._attributeGroupStyleNamespace =
			attrGroupStyleNode.getAttribute("namespace");
        for (var node = attrGroupStyleNode.firstChild;
             node != null;
             node = node.nextSibling) {
            if (node.nodeType == 1) { // If an element node ...
                if (node.tagName == "style:enumerationAttributeStyle") {
                    this._parseEnumerationAttributeStyleNode(node);
                } else if (node.tagName == "style:attributeStyle") {
                    this._parseAttributeStyleNode(node);
                }
            }
        }
    },

    _parseAttributeStyleNode: function(node) {
        var id = node.getAttribute("id");
        var displayName = node.getAttribute("displayName");
        var type = node.getAttribute("type");
        if (type) {
            switch (type) {
            case "String":
            case "Boolean":
            case "Integer":
            case "Date":
            case "Decimal":
            case "Float":
            case "URI":
				this._storeSingleFieldProperties(
					id, displayName, type);
                break;
            default:
				com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
					7203, [type, displayName]
				);
            }
        }
    },

    _parseEnumerationAttributeStyleNode: function(node) {
        var id = node.getAttribute("id");
        var displayName = node.getAttribute("displayName");
        var type = node.getAttribute("type");
        var enumNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
            node, "enumerationLiteralStyle",
            this.STYLE_NAMESPACE, this.STYLE_QUALIFIER);
        var enumValues = [];
        for (var i = 0; i < enumNodes.length; i++) {
            var enumId = enumNodes[i].getAttribute("id");
            var enumDisplayName = enumNodes[i].getAttribute("displayName");
            var enumValue = {
                id: enumId,
                displayName: enumDisplayName
            };
            enumValues.push(enumValue);
            enumValues[enumId] = enumValue;
        }
        this._storeSingleFieldProperties(
            id, displayName, "Enum", enumValues);
    },

    _storeSingleFieldProperties: function(id, displayName, type, enumValues) {
        // Note: enumValues is optional
        var singleFieldProperties = {
            id: id,
            displayName: displayName,
            type: type
        };
        if (enumValues) {
            singleFieldProperties["enumValues"] = enumValues;
        }
        this._fieldProperties[id] = singleFieldProperties;
        this._fieldProperties.push(singleFieldProperties);
    }
});

})();
