/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.rdm.web.pages.ActionViewCache");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

com.ibm.rdm.web.pages.ActionViewCache = {
    // summary:
    //    Caches the widgets shown for a particulat (JAF) action.
    // description:
    //    This object is primaryly used by ResourcesPage, which contains the  implementations
    //    of (JAF) actions. When an action is invoked (by setting window.location.href to #action=...)
    //    the corresponding function in ResourcesPage is called. As a result a widget is created and
    //    displayed. However, we do not want to reexecute the queries and recreate the widgets 
    //    when a user clicks the back/forward button of te browser (or mabe in some other cases).
    //    Browsers do not notify when back/forward was clicked, so it is not possible to know if
    //    an action was invoked via back/forward buttons, or by clicking on a link.
    //    
    //    ActionViewCache keeps a cache of the widgets used for an action. The cache is not automoatic;
    //    the methods corresponing to an action in ResourcePage (such as showProjectDashboard) need to
    //    cache the widgets themselves. ActionViewCache.cacheActionView and getCachedActionView are used
    //    by action handlers in ResourcesPage.
    //    ActionViewCache.removeCachedActionView should be called where the action is invoked (by changing
    //    window.location.href) if the cached version should not be used by the action handler (see
    //    ProjectSection for an example). If the widget removed from the cache has a destroy() method
    //    (and it should have one), it will be invoked.

    // description:
    //    _actionCache is a hash, whose key is the action name (e.g. showProjectDashboard), whose
    //    value is a list of objects, which has the following attributes:
    //    parameterHashCode: a string that uniquely represents what the view is currently showing. For example,
    //    for showProjectDashboard parameterHashCode is set to the projecy URI, and for showArtifact it is set
    //    to artifact URI.
    //    widget: This is the widget that will be cached. 
    _actionCache: {},
    
    // description:
    //    Caches a widget for a particiular state of an action. e.g.
    //    cacheActionView("showProjectDashboard", "/jazz/projects/rdm/teams/Simple", dashboard) will cache
    //    dashboard for "showProjectDashboard" action, where "/jazz/projects/rdm/teams/Simple" represents the
    //    current state of the cached widget.
    // action: String
    //    A string that represents the action, e.g. "showArtifact"
    // parameterHashCode: String
    //    A string that represents the current state of the widet that is being cached for the widget. e.g. "/artifactURI=resources/rrc71"
    // widgetToCache: dijit._Widget
    //    The widget that will be cached
    cacheActionView: function(action, parameterHashCode, widgetToCache) {
        var cacheList = this._actionCache[action];
        if (!cacheList) {
            cacheList = new Array();
            this._actionCache[action] = cacheList;
        }

        var isCached = false;
        for (var i=0; i<cacheList.length; i++) {
            if (cacheList[i].parameterHashCode==parameterHashCode) {
                ErrorHandler.log("ResourcesPage._cacheActionView() action " + action + " with parameterHashCode " + parameterHashCode + " has already been cached. Replacing existing cache.");
                cacheList.splice(i, 1, widgetToCache);
                isCached = true; 
                break;
            }
        }
        if (!isCached) {
            ErrorHandler.log("ResourcesPage._cacheActionView() action " + action + " with parameterHashCode " + parameterHashCode + " will be cached");
            cacheList.push({parameterHashCode: parameterHashCode, widget: widgetToCache});
        }
    },
    // description:
    //    Removes a widget for the provided action and state. See doc for cacheActionView method.
    removeCachedActionView: function(action, parameterHashCode) {
        var cacheList = this._actionCache[action];
        if (!cacheList)
            return;
        for (var i=0; i<cacheList.length; i++) {
            var widgetToDestroy = cacheList[i];
            if (widgetToDestroy.parameterHashCode==parameterHashCode) {
                ErrorHandler.log("ResourcesPage._cacheActionView() action " + action + " with parameterHashCode " + parameterHashCode + " has been found and will be removed from cache.");
                
                //every widget must have a properly working destroy method!
                try {
                	widgetToDestroy.widget.destroyRecursive();
                }
                catch (e) {
	            	console.trace();
	            	console.debug(widgetToDestroy);
	            	throw new Error("Every widget must have a properly working destroy method! " + e);
                }

                cacheList.splice(i, 1);
                break;
            }
        }
    },
    // description:
    //    Cheks if a widget has already been cached for the provided action and state. See doc for cacheActionView method.
    isActionViewCached:  function(action, parameterHashCode) {
        return this._getCachedActionView(action, parameterHashCode)!=null;
    },
    // description:
    //    Retrieves a widget for the provided action and state. See doc for cacheActionView method.
    getCachedActionView: function(action, parameterHashCode) {
        var cacheList = this._actionCache[action];
        if (!cacheList)
            return null;
        for (var i=0; i<cacheList.length; i++) {
            if (cacheList[i].parameterHashCode==parameterHashCode) {
                return cacheList[i]; 
            }
        }
        return null;
    }

}


})();

 