/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.FilterGroupSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.date");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.CompactItemSelectionContainer");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.project.AttributeValueSelector");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.project", "FilterGroupSection");

var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var DashboardTabPage = com.ibm.rdm.web.project.DashboardTabPage;
var constants = com.ibm.rdm.web.common.MimeTypes;


dojo.declare(
	"com.ibm.rdm.web.project.FilterGroupSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Abstract base class for each of the filter groups that appear in the
	//    "Filter Display by Attribute" section (e.g. "filter by name").

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/FilterGroupSection.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	// sectionTitle: string
	//    The title that is to appear for this filter group section.
	//    If not specified during construction, this value will be initialized
	//    to an appropriate value by reading in an appropriate NLS field.
	sectionTitle: null,
	// attributesSection: widget
	//    A pointer back to the the AttributesSection instance that originally
	//    created this filter group section.
	attributesSection: null,
    // viewType: object (representing an enum)
    //    The value of this field must be one of the values found in the
    //    com.ibm.rdm.web.project.DashboardTabPage.viewType property list.
    viewType: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_isCollapsed: null,
	_filterGroupSummaryArea: null,
	_summaryInfo: null,
	_connectHandles: null,
	_isFilterApplied: null,
	_commonMessages: null,
	_messages: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_sectionTitleSpan: null,
	_sectionSummarySpan: null,
	_removeFilterIconDiv: null,
	_contentSectionDiv: null,
	_contentDiv: null,
	_applySectionDiv: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // NLS fields requiring translation

	apply: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._sectionTitleSpan.appendChild(document.createTextNode(this.sectionTitle));
		this._isCollapsed = true;
		this._isFilterApplied = false;
		this._disableApplySection();
		this._connectHandles = [];
		this.connect(this._sectionTitleSpan, "onclick", this._onTitleMouseClick);
		this.connect(this._applyAnchor, "onclick", this._onApplyClick);
	},

    postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "FilterGroupSection");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this["apply"] = this._commonMessages["apply"];
    },

	startup: function() {
	},

	destroy: function() {
		dojo.forEach(this._connectHandles, function (handle) {
			dojo.disconnect(handle);
		});
		this.inherited(arguments);
	},

	refresh: function(refreshCompletedCallback) {
		// summary:
		//    Refresh this filter's content (if appropriate).
		//    Subclasses MAY need to override this ...
		// description:
		//    If this filter's content is based static data then nothing needs
		//    to be done.  Otherwise, if this filter's content may change, get
		//    the most up to date version of this content.
		//    For example, "Filter by name" WOULD NOT need to override this,
		//    but "Filter by user" WOULD need to override this [as a new user
		//    may have been added or removed since our last refresh].
		refreshCompletedCallback();
	},

	isFilterApplied: function() {
		return this._isFilterApplied;
	},

	isFilterSpecified: function() {
		// Subclasses need to override this ...
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection.isFilterSpecified');
	},

	clearFilter: function() {
		// Subclasses need to override this ... but they must call this super
		// version as well.
		this._removeSummaryInfo();
		this._hideRemoveFilterIcon();
		this._disableApplySection();
	},

	toggleSectionExpansionState: function() {
		// summary:
		//    If this section is currently expanded, collapse it.  Otherwise,
		//    if this section is currently collapsed, expand it.
		this._onTitleMouseClick();
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	// Helpers that subclasses need to override ...

	_createContent: function() {
		// summary:
		//    Create the content that is to be shown in this filter group.
		//    Subclasses MUST override this function.
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection._collapse');
	},

	_getSummaryInfo: function() {
		// summary:
		//    Gets a property list in a format appropriate for displaying summary
		//    information in the filter's title area.
		//    Subclasses MUST override this.
		// returns:
		//    Returns a property list in one of the three following formats:
		//    |    { type: "icons",
		//    |      iconInfos: [ {iconImgSrc: <icon image source>, iconTooltip: <icon tooltip>}* ] }
		//    or:
		//    |    { type: "single-label",
		//    |      label: <label> }
		//    or:
		//    |    { type: "multiple-labels",
		//    |      labels: [<label>*] }
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection._getSummaryInfo');
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    The filter has just been applied.
		//    Subclasses MUST override this.
		// description:
		//    Store away the current filter value(s) so that subsequent calls
		//    that ask for the current filter will use this value instead of
		//    the "intermediate value" represented by changes that may have
		//    occurred in the filter since the last time the user Apply'd this
		//    filter.
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection._filterHasBeenApplied');
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    The filter has just been removed.
		//    Subclasses MUST override this.
		// description:
		//    Reset the filter's Apply'd value back to its empty value so that
		//    subsequent calls that ask for the current filter will use this
		//    emtpy value instead of the "intermediate value" represented by
		//    changes that may have occurred in the filter since the last time
		//    the user cleared this filter.
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection._filterHasBeenRemoved');
	},

	_focus: function() {
		// summary:
		//    Focus has just been set to this filter.
		//    Subclasses MUST override this.
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.FilterGroupSection._focus');
	},

	// Helpers that don't need to be overridden ...

	_enableApplySection: function() {
		this._applySectionDiv.style.display = "";
		this._contentAreaHeightHasChanged();
	},

	_disableApplySection: function() {
		this._applySectionDiv.style.display = "none";
		this._contentAreaHeightHasChanged();
	},

	_selectionHasChanged: function() {
		// Note: Subclasses should call this when selection changes occur.
		this.attributesSection.handleSectionSelectionHasChanged();
	},

	_contentAreaHeightHasChanged: function() {
		this.attributesSection.handleContentAreaHeightHasChanged();
	},

	_onTitleMouseClick: function(event) {
		if (this._isCollapsed) {
			this._expand();
		} else {
			this._collapse();
		}
	},

	_onApplyClick: function(event) {
		dojo.stopEvent(event);
		this._invokeApplyAction();
	},

	_invokeApplyAction: function() {
		this._summaryInfo = this._getSummaryInfo();
		this._addSummaryInfo();
		this._showRemoveFilterIcon();
		this._selectionHasChanged();
		this._collapse();
	},

	_expand: function() {
		if (! this._contentDiv.firstChild) {
			this._createContent();
		}
		this._contentSectionDiv.style.display = "";
		this._isCollapsed = false;
		if (this.isFilterSpecified()) {
			this._enableApplySection();
		}
		this._focus();
		this._contentAreaHeightHasChanged();
	},

	_collapse: function() {
		this._contentSectionDiv.style.display = "none";
		this._isCollapsed = true;
		this._disableApplySection();
		this._contentAreaHeightHasChanged();
	},

	_addSummaryInfo: function() {
		var summaryInfo = this._summaryInfo;
		if (! this._filterGroupSummaryArea) {
			switch (summaryInfo.type) {
				case "icons":
					this._filterGroupSummaryArea =
						new com.ibm.rdm.web.project._IconBasedSummaryArea({
							summarySpan: this._sectionSummarySpan
						});
					break;
				case "single-label":
					this._filterGroupSummaryArea =
						new com.ibm.rdm.web.project._SingleLabelSummaryArea({
							summarySpan: this._sectionSummarySpan
						});
					break;
				case "multiple-labels":
					this._filterGroupSummaryArea =
						new com.ibm.rdm.web.project._MultipleLabelsSummaryArea({
							summarySpan: this._sectionSummarySpan
						});
					break;
			}
		}
		this._addColonSuffixToTitle();
		this._filterGroupSummaryArea.setSummaryInfo(summaryInfo);
		this._filterGroupSummaryArea.show();
	},

	_removeSummaryInfo: function() {
		if (this._filterGroupSummaryArea) {
			this._restoreOriginalSectionTitle();
			this._filterGroupSummaryArea.hide();
		}
	},

	_showRemoveFilterIcon: function() {
		if (! this._removeFilterIconDiv.firstChild) {
			var removeIcon = document.createElement("img");
			removeIcon.title = this._messages["removeFilter"];
			removeIcon.src = "com.ibm.rdm.web/common/images/remove.gif";
			removeIcon.hoverSrc = "com.ibm.rdm.web/common/images/remove-hov.gif";
			removeIcon.nohoverSrc = "com.ibm.rdm.web/common/images/remove.gif";
			this._connectHandles.push(dojo.connect(removeIcon, "onmouseover", removeIcon, function() {
				this.src = this.hoverSrc;
			}));
			this._connectHandles.push(dojo.connect(removeIcon, "onmouseout", removeIcon, function() {
				this.src = this.nohoverSrc;
			}));
			this.connect(removeIcon, "onclick", this._onRemoveClicked);
			this._removeFilterIconDiv.appendChild(removeIcon);
		}
		this._removeFilterIconDiv.style.display = "";
		this._isFilterApplied = true;
		this._filterHasBeenApplied();
	},

	_hideRemoveFilterIcon: function() {
		this._removeFilterIconDiv.style.display = "none";
		this._isFilterApplied = false;
		this._filterHasBeenRemoved();
	},

	_onRemoveClicked: function() {
		this.clearFilter();
		this._selectionHasChanged();
		this._hideRemoveFilterIcon();
	},

	_addColonSuffixToTitle: function() {
		this._setDisplayedTitle(dojo.string.substitute(this._messages.fieldName, [this.sectionTitle]));
	},

	_restoreOriginalSectionTitle: function() {
		this._setDisplayedTitle(this.sectionTitle);
	},

	_setDisplayedTitle: function(titleToUse) {
		var titleSpan = this._sectionTitleSpan;
		titleSpan.removeChild(titleSpan.firstChild);
		titleSpan.appendChild(document.createTextNode(titleToUse));
	}
});

dojo.declare(
	"com.ibm.rdm.web.project.FilterByNameSection",
	com.ibm.rdm.web.project.FilterGroupSection,
{
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_textBox: null,
	_lastAppliedArtifactNameString: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

    postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			this.sectionTitle = this._messages.filterByName;
		}
    },

	destroy: function() {
		this.inherited(arguments);
		if (this._textBox) {
			this._textBox.destroyRecursive();
		}
	},

	isFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		var isSpecified = false;
		if (this._textBox) {
			var value = this._textBox.attr('value');
			isSpecified = (value != "");
		}
		return isSpecified;
	},

	clearFilter: function() {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._textBox) {
			this._textBox.attr('value', "");
		}
	},

	getAppliedTextString: function() {
		// summary:
		//    Return the artifact name text that the user has entered.
		// returns:
		//    A string representing the possibly partial name that the user
		//    wishes to match against for filtering.
		return this._lastAppliedArtifactNameString;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_createContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var self = this;
		this._textBox = new dijit.form.TextBox({
			intermediateChanges: true,
			_onKeyPress: function(event) {
				if (dojo.trim(this.attr("value")) != "" && event.keyCode == 13) { // 13 is Enter key
					self._invokeApplyAction();
				}
				else {
					return this.inherited("_onKeyPress", arguments);
				}
			},
			onChange: function(newValue) {
 				this.inherited("onChange", arguments);
				if (dojo.trim(newValue) != "") {
					self._enableApplySection();
				} else {
					self._disableApplySection();
				}
			}
		});
		this._contentDiv.appendChild(this._textBox.domNode);
		this._textBox.startup();
	},

	_getSummaryInfo: function() {
		// summary:
		//    This is an override of a superclass function.
		return {
			type: "single-label",
			label: this._getTextString()
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedArtifactNameString = this._getTextString();
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedArtifactNameString = null;
	},

	_focus: function() {
		// summary:
		//    This is an override of a superclass function.
		if (this._textBox) {
			dijit.selectInputText(this._textBox.textbox);
		}
	},

	_getTextString: function() {
		var value = "";
		if (this._textBox) {
			value = this._textBox.attr('value');
		}
		return value;
	}
});

dojo.declare(
	"com.ibm.rdm.web.project.FilterByUserSection",
	com.ibm.rdm.web.project.FilterGroupSection,
{
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_compactItemSelectionContainer: null,
	_lastAppliedKeys: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

    postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			this.sectionTitle = this._messages.filterByUser;
		}
		this._lastAppliedKeys = [];
    },

	destroy: function() {
		this.inherited(arguments);
		if (this._compactItemSelectionContainer) {
			this._compactItemSelectionContainer.destroyRecursive();
		}
	},

	refresh: function(refreshCompletedCallback) {
		// summary:
		//    This is an override of a superclass function.
		if (this._compactItemSelectionContainer) {
			this._refresh(refreshCompletedCallback);
		} else {
			refreshCompletedCallback();
		}
	},

	isFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		var isFilterSpecified = false;
		if (this._compactItemSelectionContainer) {
			isFilterSpecified = this._compactItemSelectionContainer.isAtLeastOneItemSelected();
		}
		return isFilterSpecified;
	},

	clearFilter: function() {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._compactItemSelectionContainer) {
			this._compactItemSelectionContainer.clearSelection();
		}
	},

	getAppliedUsers: function() {
		// summary:
		//    Return an array of user names.
		// returns:
		//    An array of user names, each user name representing
		//    an a user that is to be used in "last modified by" filtering.
		//    Here's an example return value:
		//    |    ["Bob", "Kim"]
		return this._lastAppliedKeys;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_createContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var self = this;
		var selectionHasChanged = function() {
			if (self._compactItemSelectionContainer.isAtLeastOneItemSelected()) {
				self._enableApplySection();
			} else {
				self._disableApplySection();
			}
		};
		var container = new com.ibm.rdm.web.common.CompactItemSelectionContainer({
			onItemSelected: selectionHasChanged,
			onItemUnselected: selectionHasChanged,
			callbackScope: this,
			isMultiSelectionEnabled: true
		});
		this._contentDiv.appendChild(container.domNode);
		this._compactItemSelectionContainer = container;
		this._refresh();
	},

	_getSummaryInfo: function() {
		// summary:
		//    This is an override of a superclass function.
		return {
			type: "multiple-labels",
			labels: this._getSelectedUserNames()
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedKeys = this._getSelectedUserKeys();
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedKeys = [];
	},

	_focus: function() {
		// summary:
		//    This is an override of a superclass function.
		// FIXME Implement
	},

	_getSelectedUserKeys: function() {
		var userKeys = [];
		if (this._compactItemSelectionContainer
			&& this._compactItemSelectionContainer.isAtLeastOneItemSelected()
			) {
			userKeys = this._compactItemSelectionContainer.getSelectedKeys();
		}
		return userKeys;
	},
	
	_getSelectedUserNames: function() {
		var userNames = [];
		if (this._compactItemSelectionContainer
			&& this._compactItemSelectionContainer.isAtLeastOneItemSelected()
			) {
			userNames = this._compactItemSelectionContainer.getSelectedItems();
		}
		return userNames;
	},

	_refresh: function(/* Optional */ refreshCompletedCallback) {
		var self = this;
		var onComplete = function(userPropertyInfos) {
			var sortedUserPropertyInfos =
				ProjectCache.sortUserPropertiesArrayByTitle(userPropertyInfos);
			var previouslySelectedUserNames = self._compactItemSelectionContainer.getSelectedItems();
			// Clear out the old list of users.
			self._compactItemSelectionContainer.removeAllItems();
			// Add the new list of users ...
			for (var i = 0; i < sortedUserPropertyInfos.length; i++) {
				self._compactItemSelectionContainer.addTextItem(
					sortedUserPropertyInfos[i].name, sortedUserPropertyInfos[i].url);
			}
			// ... and re-select those users that were previously selected.
			if (previouslySelectedUserNames.length > 0) {
				self._compactItemSelectionContainer.setSelectedItems(previouslySelectedUserNames);
			}
			if (refreshCompletedCallback) {
				refreshCompletedCallback();
			}
			return userPropertyInfos;
		}
		var deferred = ProjectCache.getProjectUsers(this.attributesSection.projectURI);
		deferred.addCallback(this, onComplete);
	}
});

dojo.declare(
	"com.ibm.rdm.web.project.FilterByDateSection",
	com.ibm.rdm.web.project.FilterGroupSection,
{
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_fromDateTextBox: null,
	_toDateTextBox: null,
	_clearFilterRequestInProgress: null,
	_lastAppliedStartDate: null,
	_lastAppliedEndDate: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
		this._clearFilterRequestInProgress = false;
	},

    postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			this.sectionTitle = this._messages.filterByDate;
		}
    },

	destroy: function() {
		this.inherited(arguments);
		if (this._fromDateTextBox) {
			this._fromDateTextBox.destroyRecursive();
		}
		if (this._toDateTextBox) {
			this._toDateTextBox.destroyRecursive();
		}
	},

	isFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		var isSpecified = false;
		if (this._fromDateTextBox) {
			var value = this._fromDateTextBox.attr('displayedValue');
			isSpecified = (value != "");
		}
		if (!isSpecified && this._toDateTextBox) {
			var value = this._toDateTextBox.attr('displayedValue');
			isSpecified = (value != "");
		}
		return isSpecified;
	},

	clearFilter: function() {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		this._clearFilterRequestInProgress = true;
		var tomorrow = this._getTomorrow();
		if (this._fromDateTextBox) {
			this._fromDateTextBox.attr('value', null);
			this._fromDateTextBox.constraints = { max:tomorrow, selector:"date" };
			this._refreshCalendarIfNeeded(this._fromDateTextBox);
		}
		if (this._toDateTextBox) {
			this._toDateTextBox.attr('value', null);
			this._toDateTextBox.constraints = { max:tomorrow, selector:"date" };
			this._refreshCalendarIfNeeded(this._toDateTextBox);
		}
		this._clearFilterRequestInProgress = false;
	},

	getAppliedStartDate: function() {
		// summary:
		//    Get the applied start date.
		// returns:
		//    If a start date has been applied, return it as a Date instance,
		//    otherwise return null.
		return this._lastAppliedStartDate;
	},

	getAppliedEndDate: function() {
		// summary:
		//    Get the applied end date.
		// returns:
		//    If an end date has been applied, return it as a Date instance,
		//    otherwise return null.
		return this._lastAppliedEndDate;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_createContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var fromLabel = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
			"date-label",
			this._messages.startDateLabel);
		var toLabel = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
			"date-label",
			this._messages.endDateLabel);
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		var tr1 = document.createElement("tr");
		var tr2 = document.createElement("tr");
		var row1FirstTD = document.createElement("td");
		var row1SecondTD = document.createElement("td");
		var row2FirstTD = document.createElement("td");
		var row2SecondTD = document.createElement("td");
		var self = this;
		// TODO Move these DataTextBox width specifications to css?
		var tomorrow = this._getTomorrow();
		var fromDateTextBox = new dijit.form.DateTextBox({
			style: "width: 100px;",
			constraints: { max:tomorrow, selector:"date" },
			onChange: function() {
				self._handleDateChanged(this);
			}
		});
		var toDateTextBox = new dijit.form.DateTextBox({
			style: "width: 100px;",
			constraints: { max:tomorrow, selector:"date" },
			onChange: function() {
				self._handleDateChanged(this);
			}
		});
		// Hook up the table
		table.appendChild(tbody);
		tbody.appendChild(tr1);
		tbody.appendChild(tr2);
		tr1.appendChild(row1FirstTD);
		tr1.appendChild(row1SecondTD);
		tr2.appendChild(row2FirstTD);
		tr2.appendChild(row2SecondTD);
		row1FirstTD.appendChild(fromLabel);
		row1SecondTD.appendChild(fromDateTextBox.domNode);
		row2FirstTD.appendChild(toLabel);
		row2SecondTD.appendChild(toDateTextBox.domNode);
		// Set fields
		this._fromDateTextBox = fromDateTextBox;
		this._toDateTextBox = toDateTextBox;
		// Hook up this new table to the content div
		this._contentDiv.appendChild(table);
		// Startup appropriate widgets
		this._fromDateTextBox.startup();
		this._toDateTextBox.startup();
	},

	_getSummaryInfo: function() {
		// summary:
		//    This is an override of a superclass function.
		var label = "";
		var startDate = this._fromDateTextBox.attr('displayedValue');
		if (this._toDateTextBox.attr('displayedValue')) {
			var endDate = this._toDateTextBox.attr('displayedValue');
			label = dojo.string.substitute(
				this._messages.dateSummaryLabelUsingStartAndEndDate,[startDate, endDate]);
		}
		else {
			label = dojo.string.substitute(
				this._messages.dateSummaryLabelUsingStartDate, [startDate]);
		}
		return {
			type: "single-label",
			label: label
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedStartDate = this._getStartDate();
		this._lastAppliedEndDate = this._getEndDate();
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedStartDate = null;
		this._lastAppliedEndDate = null;
	},

	_focus: function() {
		// summary:
		//    This is an override of a superclass function.
		if (this._fromDateTextBox) {
			this._fromDateTextBox.focus();
		}
	},

	_getStartDate: function() {
		// summary:
		//    Get the start date.
		// returns:
		//    If a start date has been specified, return it as a Date instance,
		//    otherwise return null.
		var value = null;
		if (this._fromDateTextBox) {
			value = this._fromDateTextBox.attr('value');
		}
		return value;
	},

	_getEndDate: function() {
		// summary:
		//    Get the end date.
		// returns:
		//    If an end date has been specified, return it as a Date instance,
		//    otherwise return null.
		var value = null;
		if (this._toDateTextBox) {
			value = this._toDateTextBox.attr('value');
		}
		return value;
	},

	_getToday: function() {
		return (new Date()).setUTCHours(0, 0, 0, 0);
	},

	_getTomorrow: function() {
		return dojo.date.add(this._getToday(), "day", 1);
	},

	_handleDateChanged: function(dateTextBox) {
		if (! this._clearFilterRequestInProgress) {
			if (dateTextBox === this._fromDateTextBox) {
				// The 'From' date has changed.
				// Invalidate dates BEFORE this one in the To field's calendar.
				this._toDateTextBox.constraints.min = this._fromDateTextBox.attr('value');
				this._refreshCalendarIfNeeded(this._toDateTextBox);
			} else {
				// The 'To' date has changed.
				// Invalidate dates AFTER this one in the From field's calendar.
				this._fromDateTextBox.constraints.max =
					dojo.date.add((this._toDateTextBox.attr('value')), "day", 1);
				this._refreshCalendarIfNeeded(this._fromDateTextBox);
			}
			// Only enable Apply if there's a From date or a From date and a
			// To date.
			if (this._fromDateTextBox.attr('value')
				|| this._toDateTextBox.attr('value') && this._fromDateTextBox.attr('value')
				) {
				this._enableApplySection();
			} else {
				this._disableApplySection();
			}
		}
	},

	_refreshCalendarIfNeeded: function(dateTextBox) {
		calendar = dateTextBox._picker;
		//
		// TODO Report this bug to dojo ...
		//
		// Note: We're working around a dojo bug here ... if the constraint.min
		// or constraint.max is adjusted AFTER the calendar has been displayed
		// at least once, for some reason the calendar does NOT add strike-
		// throughs to the dates that should now be classified as invalid
		// (but if you view previous month and then next month, the display
		// DOES get updated with the new invalidated dates).
		if (calendar) {
			calendar._populateGrid();
		}
	}
});

dojo.declare(
	"com.ibm.rdm.web.project.FilterByArtifactTypeSection",
	com.ibm.rdm.web.project.FilterGroupSection,
{
	///////////////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_compactItemSelectionContainer: null,
	_lastAppliedArtifactTypes: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

    postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			this.sectionTitle = this._messages.filterByArtifactType;
		}
    },

	destroy: function() {
		this.inherited(arguments);
		if (this._compactItemSelectionContainer) {
			this._compactItemSelectionContainer.destroyRecursive();
		}
	},

	isFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		var isFilterSpecified = false;
		if (this._compactItemSelectionContainer) {
			isFilterSpecified = this._compactItemSelectionContainer.isAtLeastOneItemSelected();
		}
		return isFilterSpecified;
	},

	clearFilter: function() {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._compactItemSelectionContainer) {
			this._compactItemSelectionContainer.clearSelection();
		}
	},

	getAppliedArtifactTypes: function() {
		// summary:
		//    Return an array of content types, each content type representing
		//    an artifact type that has been selected.
		return this._lastAppliedArtifactTypes;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_createContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var self = this;
		var selectionHasChanged = function() {
			if (self._compactItemSelectionContainer.isAtLeastOneItemSelected()) {
				self._enableApplySection();
			} else {
				self._disableApplySection();
			}
		};
		// TODO As a performance improvement we might want to sort the items
		// _before_ making calls to 'addTextItem' (as sorting here would
		// probably be faster than asking the container to place each new item
		// in the appropriate sorted position).  If we were to do this we
		// would pass 'false' for 'doInsertItemsInSortedOrder' in the
		// following call ....
		var container = new com.ibm.rdm.web.common.CompactItemSelectionContainer({
			onItemSelected: selectionHasChanged,
			onItemUnselected: selectionHasChanged,
			callbackScope: this,
			isMultiSelectionEnabled: true,
			doInsertItemsInSortedOrder: true
		});
		this._addComposerArtifactTypes(container);
		this._addWrappedArtifactTypes(container);
		this._contentDiv.appendChild(container.domNode);
		this._compactItemSelectionContainer = container;
	},

	_getSummaryInfo: function() {
		// summary:
		//    This is an override of a superclass function.
		// returns:
		//    Returns a property array in the following format:
		//    |    { type: "multiple-labels",
		//    |      labels: [<label>*] }
		var selectedTypes = [];
		if (this._compactItemSelectionContainer
			&& this._compactItemSelectionContainer.isAtLeastOneItemSelected()
			) {
			selectedTypes = this._compactItemSelectionContainer.getSelectedItems();
		}
		return {
			type: "multiple-labels",
			labels: selectedTypes
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedArtifactTypes = this._getArtifactTypes();
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedArtifactTypes = null;
	},

	_focus: function() {
		// summary:
		//    This is an override of a superclass function.
		// FIXME Implement
	},

	_getArtifactTypes: function() {
		// summary:
		//    Get the selected artifact types.
		// returns:
		//    Returns an array of content types, each content type representing
		//    an artifact type that has been selected.
		var selectedContentTypes = [];
		if (this._compactItemSelectionContainer
			&& this._compactItemSelectionContainer.isAtLeastOneItemSelected()
			) {
			// Note: All items were originally created by passing the
			// 'itemKey' param to 'CompactItemSelectionContainer.addTextItem'
			// and each of these keys specified the associated content type
			// (or array of content types in the case of such types as Image
			// or Video).
			var selectedItemKeyValues = this._compactItemSelectionContainer.getSelectedKeys();
			for (var i = 0; i < selectedItemKeyValues.length; i++) {
				var keyValue = selectedItemKeyValues[i];
				if (dojo.isArray(keyValue)) {
					var contentTypes = keyValue;
					for (var j = 0; j < contentTypes.length; j++) {
						selectedContentTypes.push(contentTypes[j]);
					}
				} else {
					selectedContentTypes.push(keyValue);
				}
			}
		}
		return selectedContentTypes;
	},

	_addComposerArtifactTypes: function(container) {
		container.addTextItem(
			this._commonMessages["glossary"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY);
		container.addTextItem(
			this._commonMessages["term"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM);
		container.addTextItem(
			this._commonMessages["process"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS);
		container.addTextItem(
			this._commonMessages["useCaseDiagram"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM);
		container.addTextItem(
			this._commonMessages["useCase"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE);
		container.addTextItem(
			this._commonMessages["actor"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_ACTOR);
		container.addTextItem(
			this._commonMessages["part"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART);
		container.addTextItem(
			this._commonMessages["sketch"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH);
		container.addTextItem(
			this._commonMessages["storyBoard"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD);
		container.addTextItem(
			this._commonMessages["screenFlow"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW);
		container.addTextItem(
			this._commonMessages["richTextDocument"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT);
		container.addTextItem(
			this._commonMessages["requirement"], com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT);
	},

	_addWrappedArtifactTypes: function(container) {
		var imageTypes = [
			constants.JPEG.contentType,
			constants.GIF.contentType,
			constants.PNG.contentType,
			constants.ICO.contentType,
			constants.BMP.contentType
		];
		var videoTypes = [
			constants.MP4.contentType,
			constants.AVI.contentType,
			constants.MV.contentType,
			constants.MOV.contentType,
			constants.MPEG.contentType
		];
		container.addTextItem(
			this._commonMessages["zipFile"], constants.ZIP.contentType);
		container.addTextItem(
			this._commonMessages["xmlFile"], constants.XML.contentType);
		container.addTextItem(
			this._commonMessages["htmlFile"], constants.HTML.contentType);
		container.addTextItem(
			this._commonMessages["image"], imageTypes);
		container.addTextItem(
			this._commonMessages["javascriptFile"], constants.JS.contentType);
		container.addTextItem(
			this._commonMessages["microsoftExcelDocument"], constants.XLS.contentType);
		container.addTextItem(
			this._commonMessages["microsoftPowerPointDocument"], constants.PPT.contentType);
		container.addTextItem(
			this._commonMessages["microsoftProjectDocument"], constants.PRJ.contentType);
		container.addTextItem(
			this._commonMessages["microsoftWordDocument"], constants.DOC.contentType);
		container.addTextItem(
			this._commonMessages["pdfDocument"], constants.PDF.contentType);
		container.addTextItem(
			this._commonMessages["plainText"], constants.TXT.contentType);
		container.addTextItem(
			this._commonMessages["ravenFlow"], constants.RAVENFLOW.contentType);
		container.addTextItem(
			this._commonMessages["realTimeTransportProtocol"], constants.RTP.contentType);
		container.addTextItem(
			this._commonMessages["tapeArchive"], constants.TAR.contentType);
		container.addTextItem(
			this._commonMessages["texFormat"], constants.TEX.contentType);
		container.addTextItem(
			this._commonMessages["video"], videoTypes);
		container.addTextItem(
			this._commonMessages["iRiseSimulation"], constants.IRIS.contentType);
		// FIXME Hunt down what "Other" means in native client and add that
		// item as well.
	}
});

dojo.declare(
	"com.ibm.rdm.web.project.FilterByAttributeValueSection",
	com.ibm.rdm.web.project.FilterGroupSection,
{
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_attributeValueSelector: null,
	_lastAppliedAttributeValuesInfo: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			this.sectionTitle = this._messages.filterByAttribute;
		}
	},

	destroy: function() {
		this.inherited(arguments);
		if (this._attributeValueSelector) {
			this._attributeValueSelector.destroyRecursive();
		}
	},

	refresh: function(refreshCompletedCallback) {
		// summary:
		//    This is an override of a superclass function.
		var self = this;
		if (this._attributeValueSelector) {
			var onAttributeGroupsRefreshCompleted = function() {
				// Note: After obtaining "fresh" attribute group info it MAY be
				// the case that the current filter is no longer valid.  For
				// example, if the filter is based on an enum field value,
				// perhaps this field value has been removed.  If the filter is
				// no longer valid we take care here to remove the filter BEFORE
				// telling the DashboardTabPage (via refreshCompletedCallback)
				// that we're up to date (as we want to make sure that this
				// invalid filter DOES NOT get added to the query that is
				// issued).
				var isLastAppliedAttributeValuesStillValid = (
					self._lastAppliedAttributeValuesInfo
					&& self._attributeValueSelector.isAttributeValuesInfoStillValid(
						self._lastAppliedAttributeValuesInfo)
				);
				if (self._lastAppliedAttributeValuesInfo != null
					&& (! isLastAppliedAttributeValuesStillValid)
					) {
					self.clearFilter();
				}
				refreshCompletedCallback();
			}
			this._attributeValueSelector.refresh(onAttributeGroupsRefreshCompleted);
		} else {
			refreshCompletedCallback();
		}
	},

	isFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		var isFilterSpecified = false;
		if (this._attributeValueSelector) {
			isFilterSpecified = this._attributeValueSelector.isAttributeValueSpecified();
		}
		return isFilterSpecified;
	},

	clearFilter: function() {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._attributeValueSelector) {
			this._attributeValueSelector.clearSelection();
		}
	},

	getAppliedAttributeValuesInfo: function() {
		return this._lastAppliedAttributeValuesInfo;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_createContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var self = this;
		var attributeValueSelector = new com.ibm.rdm.web.project.AttributeValueSelector({
			projectURI: this.attributesSection.projectURI,
			onInvokeApplyAction: function() { self._invokeApplyAction(); },
			onEnableApplySection: function() { self._enableApplySection(); },
			onDisableApplySection: function() { self._disableApplySection(); },
			onContentAreaHeightHasChanged: function() { self._contentAreaHeightHasChanged(); }
		});
		this._contentDiv.appendChild(attributeValueSelector.domNode);
		this._attributeValueSelector = attributeValueSelector;
	},

	_getSummaryInfo: function() {
		// summary:
		//    This is an override of a superclass function.
		label = "";
		if (this._attributeValueSelector) {
			label = this._attributeValueSelector.getSummaryLabel();
		}
		return {
			type: "single-label",
			label: label
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedAttributeValuesInfo = this._getAppliedAttributeValuesInfo();
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//    This is an override of a superclass function.
		this._lastAppliedAttributeValuesInfo = null;
	},

	_focus: function() {
		// summary:
		//    This is an override of a superclass function.
		if (this._attributeValueSelector) {
			this._attributeValueSelector.focus();
		}
	},

	_getAppliedAttributeValuesInfo: function() {
		var attributeValuesInfo = null;
		if (this._attributeValueSelector) {
			attributeValuesInfo = this._attributeValueSelector.getAttributeValuesInfo();
		}
		return attributeValuesInfo;
	}
});

dojo.declare(
	"com.ibm.rdm.web.project._FilterGroupSummaryArea",
	dijit._Widget,
{
	// summary:
	//    Abstract base class for a "summary area" that appears to the right
	//    of the filter group titles (e.g. to the right of "Filter by name").

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields
	
	// summarySpan: DOM node
	//     A SPAN node in which to present summary info.
	summarySpan: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	setSummaryInfo: function(summaryInfo) {
		// Subclasses need to override this ...
		// summaryInfo: property list
		//    A property list of one of the following three formats:
		//    |    { type: "icons",
		//    |      iconInfos: [ {iconImgSrc: <icon image source>, iconTooltip: <icon tooltip>}* ] }
		//    or:
		//    |    { type: "single-label",
		//    |      label: <label> }
		//    or:
		//    |    { type: "multiple-labels",
		//    |      labels: [<label>*] }
	},

	show: function() {
		this.summarySpan.style.display = "";
	},

	hide: function() {
		this.summarySpan.style.display = "none";
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_clear: function() {
		var span = this.summarySpan;
		while (span.firstChild) {
			span.removeChild(span.firstChild);
		}
	}
});

dojo.declare(
	"com.ibm.rdm.web.project._IconBasedSummaryArea",
	com.ibm.rdm.web.project._FilterGroupSummaryArea,
{
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	setSummaryInfo: function(summaryInfo) {
		// summary:
		//    This is an override of a superclass function.
		// summaryInfo: property list
		//    A property list of the following format:
		//    |    { type: "icons",
		//    |      iconInfos: [ {iconImgSrc: <icon image source>, iconTooltip: <icon tooltip>}* ] }
		this._clear();
		var iconInfos = summaryInfo.iconInfos;
		for (var i = 0; i < iconInfos.length; i++) {
			var iconImgSrc = iconInfos[i].iconImgSrc;
			// TODO Add tooltips as well ...
			//var iconTooltip = iconInfos[i].iconTooltip;
            var img = document.createElement("img");
            img.setAttribute("src", iconImgSrc);
			this.summarySpan.appendChild(img);
		}
	}

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
});

dojo.declare(
	"com.ibm.rdm.web.project._MultipleLabelsSummaryArea",
	com.ibm.rdm.web.project._FilterGroupSummaryArea,
{
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	setSummaryInfo: function(summaryInfo) {
		// summary:
		//    This is an override of a superclass function.
		// summaryInfo: property list
		//    A property list of the following format:
		//    |    { type: "multiple-labels",
		//    |      labels: [<label>*] }
		this._clear();
		var labels = summaryInfo.labels;
		for (var i = 0; i < labels.length; i++) {
			var labelToUse = (
				(i < (labels.length-1)) ?
				(summaryInfo.labels[i] + ", ") :
				summaryInfo.labels[i]
			);
			this.summarySpan.appendChild(
				document.createTextNode(labelToUse));
		}
	}

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
});

dojo.declare(
	"com.ibm.rdm.web.project._SingleLabelSummaryArea",
	com.ibm.rdm.web.project._FilterGroupSummaryArea,
{
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	setSummaryInfo: function(summaryInfo) {
		// summary:
		//    This is an override of a superclass function.
		// summaryInfo: property list
		//    A property list of the following format:
		//    |    { type: "single-label",
		//    |      label: <label> }
		this._clear();
		this.summarySpan.appendChild(
			document.createTextNode(summaryInfo.label));
	}

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
});

})();
