/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.TagsSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.common.CompactItemSelectionContainer");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.project.DashboardSection");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");

var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

dojo.declare(
    "com.ibm.rdm.web.project.TagsSection",
    com.ibm.rdm.web.project.DashboardSection,
{
    // summary:
    //    Presents a section view on the left in the project dashboard to
    //    enable users to filter by tags.

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields
    projectURI: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

	_tagsDiv: null,
	_publicTagsContainer: null,
	_privateTagsContainer: null,
    
    // This attribute will ONLY hold a value when a refresh operation is in
    // progress.  (Note: This attribute is null'd out in the constructor.)
    _refreshInfo: {
        // The callback to invoke when the refresh operation has completed (i.e.
        // after new values have been added to the DOM that reflect a refreshed
        // tag list).
        refreshCompletedCallback: null
    },
    _publicRefreshCompleted: false,
    _privateRefreshCompleted: false,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    constructor: function() {
        this._refreshInfo = null;
    },

    postCreate: function() {
        this.inherited("postCreate", arguments);
		this._publicTagsContainer =
			new com.ibm.rdm.web.common.CompactItemSelectionContainer({
				onItemSelected: this._onTagSelectionChanged,
				onItemUnselected: this._onTagSelectionChanged,
				callbackScope: this,
				isMultiSelectionEnabled: true,
				textItemCssClassName: "public-tag-name"
		});
		this._privateTagsContainer =
			new com.ibm.rdm.web.common.CompactItemSelectionContainer({
				onItemSelected: this._onTagSelectionChanged,
				onItemUnselected: this._onTagSelectionChanged,
				callbackScope: this,
				isMultiSelectionEnabled: true,
				textItemCssClassName: "private-tag-name"
		});
		this._tagsDiv = document.createElement("div");
		this._tagsDiv.appendChild(this._publicTagsContainer.domNode);
		var separatorDiv = document.createElement("div");
        separatorDiv.className = "tag-name-separator";
		this._tagsDiv.appendChild(separatorDiv);
		this._tagsDiv.appendChild(this._privateTagsContainer.domNode);
        this._contentAreaDiv.appendChild(this._tagsDiv);
    },

    startup: function() {
        this.inherited("startup", arguments);
    },

    destroy: function() {
        this.inherited(arguments);
        this._publicTagsContainer.destroyRecursive();
        this._privateTagsContainer.destroyRecursive();
    },

    getSelectedTags: function() {
        // summary:
        //    Return an array of strings where each element of the array
        //    represents the resource URI for a selected tag.
		var selectedTagURIs = [];
		if (this._publicTagsContainer) {
			selectedTagURIs = selectedTagURIs.concat(
				this._publicTagsContainer.getSelectedKeys(),
				this._privateTagsContainer.getSelectedKeys());
		}
		return selectedTagURIs;
    },

    refresh: function(refreshCompletedCallback) {
        this._startRefresh(refreshCompletedCallback);
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    // Superclass override functions ...

    _populateContents: function() {
        // Note: this is an override of a superclass function ...
		var publicTagURIs = this._publicTagsContainer.getSelectedKeys();
		var privateTagURIs = this._privateTagsContainer.getSelectedKeys();
    	this._clearLastResultsIfNeeded();
		this._setContentToLoadingMessage();
        this._deferredArray = [];

		var deferred = ProjectCache.getProjectTags(this.projectURI);
		deferred.addCallback(this, function(tagsMap) {
			var tagNameComparator = function(resource1, resource2) {
				return ((resource1.title == resource2.title) ? 0 : 
					((resource1.title.toLowerCase() < resource2.title.toLowerCase()) ? -1 : 1));
			};
			
			var tags = [];
			//convert tags map to array
			for (var tagResourceURI in tagsMap) {
				tags.push(tagsMap[tagResourceURI]);
			}
			tags.sort(tagNameComparator);
			
			this._setContentToTagResults();
			for (var i = 0; i < tags.length; i++ ) {
				var tagName = tags[i].title;
				var tagURI = tags[i].resourceURI;
				(tags[i].scope=='public'?this._publicTagsContainer:this._privateTagsContainer).addTextItem(tagName, tagURI);
			}
			this._publicTagsContainer.setSelectedKeys(publicTagURIs);
			this._privateTagsContainer.setSelectedKeys(privateTagURIs);
			this._publicRefreshCompleted = true;
			this._privateRefreshCompleted = true;
			this._endRefreshIfNeeded();
			return tags;
		});
    },

    _handleClearFilters: function() {
        // Note: this is an override of a superclass function ...
		this._publicTagsContainer.clearSelection();
		this._privateTagsContainer.clearSelection();
        this._tagSelectionHasChanged();
    },

	_isAtLeastOneFilterSpecified: function() {
		return (this.getSelectedTags().length > 0);
	},

    // Normal (non-override) functions ...

	_onTagSelectionChanged: function() {
		// summary:
		//    This function is called whenever an item becomes selected or
		//    unselected in either the public or private tags container.
		this._tagSelectionHasChanged();
	},

	_setContentToLoadingMessage: function() {
		if (this._contentAreaDiv.firstChild === this._tagsDiv) {
			this._contentAreaDiv.removeChild(this._tagsDiv);
		}
		this._contentAreaDiv.innerHTML = "";
		this.showLoadingMessage();
	},

	_setContentToTagResults: function() {
		if (this._contentAreaDiv.firstChild !== this._tagsDiv) {
			this._contentAreaDiv.innerHTML = "";
			this._contentAreaDiv.appendChild(this._tagsDiv);
		}
	},

    _startRefresh: function(refreshCompletedCallback) {
        // To handle the case of a refresh being asked for before the last one
        // has completed, we'll ignore subsequent requests if one is currently
        // under way.
        if (!this._refreshInfo) {
        	// reset refresh flags
        	this._publicRefreshCompleted = false;
        	this._privateRefreshCompleted = false;
            this._refreshInfo = {
                refreshCompletedCallback: refreshCompletedCallback
            };
            this._populateContents();
        }
    },

    _endRefreshIfNeeded: function() {
        if (this._refreshInfo) {
        	if (this._publicRefreshCompleted && this._privateRefreshCompleted) {
	            var callback = this._refreshInfo.refreshCompletedCallback;
	            this._refreshInfo = null;
	            callback();
        	}
        }
    },

    _clearLastResultsIfNeeded: function() {
		this._publicTagsContainer.removeAllItems();
		this._privateTagsContainer.removeAllItems();
    },

    _tagSelectionHasChanged: function() {
		this._filterSelectionHasChanged();
		this.dashboardTabPage.refresh();
    }
});

})();