/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.reviewutils");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.review.reviewTypes");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


com.ibm.rdm.web.review.reviewutils = {
	canVote: function(reviewParser) {
		// summary:
		//    Returns true iff the current user should be able to cast votes
		//    within the review represented by reviewParser.
		var isDone = function() {
			var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
			var participantResultInfos = reviewParser.getParticipantResultInfos();
			var myParticipantResultInfo = participantResultInfos[myUserURI];
			return (
				myParticipantResultInfo ?
				myParticipantResultInfo.artifactStatusType :
				false
			);
		};
		return (
			this.isVoteCastingPermissionFoundInUserRole(reviewParser)
			&& reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Started
			&& reviewParser.getRole()
			&& (! isDone())
		);
	},

	isVoteCastingPermissionFoundInUserRole: function(reviewParser) {
		// summary:
		//    Returns true iff the current user's account role specifies that
		//    the review vote casting permission has been enabled.
		var projectOperations =
		    com.ibm.rdm.web.AUTHENTICATED_USER.permissions.item(reviewParser.getProjectUri());
		return projectOperations.item(
			com.ibm.rdm.web.common.ProjectCache.MODIFY_REVIEW_APPROVAL
		);
	},

	createArtifactStatusTypeImage: function(artifactStatusType, /*String?*/cssClassName) {
		// summary:
		//    Create an 'img' that can be used to represent the review status
		//    for a particular artifact under review.
		// returns: DomNode
		//    A newly created 'img'.
		// artifactStatusType:
		//    One of the values found in reviewTypes.ARTIFACT_STATUS_TYPES
		//    (e.g. reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted).
		var img = document.createElement("img");
		img.title = reviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType);
		img.src = reviewTypes.getArtifactStatusTypeImageURI(artifactStatusType);
		if (cssClassName) {
			img.className = cssClassName;
		}
		return img;
	},

	createReviewStateImage: function(reviewStateType, /*String?*/cssClassName) {
		// summary:
		//    Create an 'img' that can be used to represent the review state.
		// returns: DomNode
		//    A newly created 'img'.
		// reviewStateType:
		//    One of the values found in reviewTypes.REVIEW_STATE_TYPES (e.g.
		//    reviewTypes.REVIEW_STATE_TYPES.Started).
		var img = document.createElement("img");
		img.title = reviewTypes.getReviewStateDisplayName(reviewStateType);
		img.src = reviewTypes.getReviewStateTypeImageURI(reviewStateType);
		if (cssClassName) {
			img.className = cssClassName;
		}
		return img;
	},

	createParticipantRoleImage: function(participantRoleType) {
		// summary:
		//    Create an 'img' that can be used to represent a participant's role.
		// returns: DomNode
		//    A newly created 'img'.
		// participantRoleType:
		//    One of the values found in reviewTypes.PARTICIPANT_ROLE_TYPES (e.g.
		//    reviewTypes.PARTICIPANT_ROLE_TYPES.Approver).
		var img = document.createElement("img");
		img.title = reviewTypes.getParticipantRoleTypeDisplayName(participantRoleType);
		img.src = reviewTypes.getParticipantRoleTypeImageURI(participantRoleType);
		return img;
	},

	resourceIdToResourceURI: function(resourceId, reviewParser) {
		// summary:
		//    Given a resource ID like "_7nOg4DvdEd6IT-BLBFfrNQ", return
		//    a resource URI like "https://localhost:9443/rdm/resources/_7nOg4DvdEd6IT-BLBFfrNQ".
		var reviewURI = reviewParser.getSelfUri();
		var lastSlashIndex = reviewURI.lastIndexOf("/");
		return (
			reviewURI.substring(0, lastSlashIndex+1)
			+ resourceId
		);
	},

	composeLastModifiedDateDisplayLabel: function(/*Date*/date) {
		var dateStr = dojo.date.locale.format(
			date, { formatLength:"medium", selector:"date" }
		);
		var timeStr = dojo.date.locale.format(
			date, { formatLength:"short", selector:"time" }
		);
		return (dateStr + " " + timeStr);
	},

	fillUserSpanAndAddUserImage: function(userURI, userSpan, /*Callback?*/onUserInfoComplete) {
		// summary:
		//    Fill span element userSpan with the appropriate user name and add
		//    a user image before this span element (both pertaining to the
		//    passed userURI).
		// onUserInfoComplete: 
		//    If passed, this callback will be called with two arguments;
		//    userInfo and userSpan.  The userInfo arg represents the userInfo
		//    that is associated with the passed userURI and the userSpan arg
		//    represents the the newly created span element that holds the user
		//    name.
		var onAllUsersComplete = function(mapUserURIToUserInfo) {
			var userInfo = mapUserURIToUserInfo[userURI];
			var username = (userInfo ? userInfo.name : userURI);
			var photoURI = (
				userInfo ?
				userInfo.photoUri :
				"com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
			);
			var img = document.createElement("img");
			img.className = "userImage";
			img.src = photoURI;
			userSpan.innerHTML = username;
			dojo.place(img, userSpan, "before");
			if (onUserInfoComplete) {
				onUserInfoComplete(userInfo, userSpan);
			}
			return mapUserURIToUserInfo;
		};
		ProjectCache.getAllUsers().addCallback(this, onAllUsersComplete);
	}
};

})();
